package fishrungames.yelpmapapp

import android.location.Location
import android.os.Handler
import com.google.android.gms.maps.GoogleMap
import com.google.android.gms.maps.model.Marker
import com.google.android.gms.maps.model.BitmapDescriptorFactory
import com.google.android.gms.maps.model.MarkerOptions
import com.google.android.gms.maps.model.LatLng
import java.util.*
import com.google.maps.android.clustering.ClusterManager
import com.google.android.gms.maps.CameraUpdateFactory






/**
 * Created by mephi on 09.06.2017.
 */


class YelpMapActivity : MapsActivity()
{

    val handler = Handler()

    val dataProvider = DataProvider(this, this::handleMarkersUpdated)

    private var mClusterManager: ClusterManager<MapMarkerClusterItem>? = null

    val markerMap = mutableMapOf<String, MapMarkerClusterItem>()

    var latestRequestDate : Date? = null

    var requestScheduled : Boolean = false

    override fun onMapReady(googleMap: GoogleMap) {
        super.onMapReady(googleMap)

        googleMap.setOnCameraMoveListener {

            tryRequestData(googleMap.cameraPosition.target.latitude, googleMap.cameraPosition.target.longitude)

        }

        setUpClusterer(googleMap)
    }


    override fun onLocationChanged(location: Location) {
        super.onLocationChanged(location)
        tryRequestData(location.latitude, location.longitude)
    }

    fun handleMarkersUpdated(newMarkerMap: Map<String, MapMarkerRecord>)
    {
        if (mClusterManager == null)
        {
            return
        }

        val clusterManager = mClusterManager!!

        for (key in newMarkerMap.keys)
        {


            if (!markerMap.containsKey(key))
            {
                val mapMarkerRecord = newMarkerMap[key] as MapMarkerRecord

                val mapMarkerClusterItem = MapMarkerClusterItem(mapMarkerRecord)

                clusterManager.addItem(mapMarkerClusterItem)

                markerMap[key] = mapMarkerClusterItem

            }
        }
    }



    fun tryRequestData(lat : Double, lon : Double) {

        val requestLimitInMs = 2000L

        val newDate = Date()

        if (latestRequestDate == null)
        {
            innerRequestData(lat, lon)
        }
        else {
            if (!requestScheduled) {

                requestScheduled = true

                val timeDiffMs = newDate.time - latestRequestDate!!.time

                if (timeDiffMs > requestLimitInMs) {
                    innerRequestData(lat, lon)
                } else {

                    handler.postDelayed({ innerRequestData(lat, lon) }, requestLimitInMs - timeDiffMs)

                }
            }
        }
    }

    fun innerRequestData(lat : Double, lon : Double)
    {
        val radius = 2000

        latestRequestDate = Date()

        dataProvider.requestData(lat, lon, radius)

        requestScheduled = false

    }


    private fun setUpClusterer(googleMap: GoogleMap) {

        // Initialize the manager with the context and the map.
        // (Activity extends context, so we can pass 'this' in the constructor.)
        mClusterManager = ClusterManager<MapMarkerClusterItem>(this, googleMap)

        // Point the map's listeners at the listeners implemented by the cluster
        // manager.
        googleMap.setOnCameraIdleListener(mClusterManager)
        googleMap.setOnMarkerClickListener(mClusterManager)

    }


}